#!/usr/bin/python

import time
from collections import deque
import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as ds
import datetime as dt
import matplotlib.dates as mdates
import glob
import sys

def draw_graph(t,ir,Y,m,d,M,H,S,imgfile):

    t = np.array(t)
    ir = np.array(ir)

    times = np.zeros(len(Y))
    for i in range(0,len(Y)):
        times[i] = ds.date2num(dt.datetime(Y[i],m[i],d[i],H[i],M[i],S[i]))

    ok = np.where(ir > -50)
    t = t[ok]
    ir = ir[ok]
    times = times[ok]

    ok = np.where(t > -50)
    t = t[ok]
    ok2 = np.where(t < 50)
    t = t[ok2]
    ir = ir[ok]
    times = times[ok]
    times2 = times[ok2]

    xmin = times.min()
    xmax = times.max()

    xFmt = mdates.DateFormatter('%d')
    days = mdates.DayLocator()
    hours = mdates.HourLocator(interval=6)

    width = 1200/100.0
    height = 400/100.0
    
    fig = plt.figure(figsize=(width,height))
    ax = fig.add_subplot(111)

#    ax.plot(times,t,'r.',markersize=3.0)
#    ax.plot(times,ir,'b.',markersize=3.0)
    ax.plot(times2,t,'r')
    ax.plot(times,ir,'b')

    # format the ticks
    ax.xaxis.set_major_locator(days)
    ax.xaxis.set_major_formatter(xFmt)
    ax.xaxis.set_minor_locator(hours)

    ax.set_xlim(xmin,xmax)
    ax.grid(True)

    t_max = t.max()+1
    ir_max = ir.max()+1
    
    leg = plt.legend(('Ambient','Sky'), loc=2)
    leg.get_frame().set_alpha(0.5)
    plt.ylabel('Temperature $^{\circ}$C')
    plt.xlabel('Day of month')
    ax.set_title('Sky and ambient temperature in Perkkaa, '+str(min(d))+'.-'+str(max(d))+'.'+str(m[0])+'.'+str(Y[0]))

#    ax.spines['right'].set_color((1,1,1,0))
    for t in ax.xaxis.get_ticklines(): t.set_color('gray')
    for t in ax.yaxis.get_ticklines(): t.set_color('gray')

    plt.savefig(imgfile,format='png')


datadir = ''
imgdir = ''
#datadir = '/tmp'
#imgdir = '/tmp'

year = sys.argv[1]
month = sys.argv[2]

temps = deque()
ir_temps = deque()
Y = deque()
m = deque()
d = deque()
H = deque()
M = deque()
S = deque()

files = glob.glob(datadir+'/'+str(year)+str(month)+'*_cloud_sensor.txt')
files.sort()

print 'Reading files'
for f in files:
    try:
        data = np.loadtxt(f, delimiter=',')
        for i in range(0,len(data)):
            temps.append(data[i,0].astype('float'))
            ir_temps.append(data[i,1].astype('float'))
            Y.append(data[i,2].astype('int'))
            m.append(data[i,3].astype('int'))
            d.append(data[i,4].astype('int'))
            H.append(data[i,5].astype('int'))
            M.append(data[i,6].astype('int'))
            S.append(data[i,7].astype('int'))
    except IOError:
        pass

print 'Drawing monthly plot'
imgfile=imgdir+'/'+str(year)+str(month)+'_cloud_sensor_monthly.png'
draw_graph(temps,ir_temps,Y,m,d,M,H,S,imgfile)
    

