#!/usr/bin/python

import sys
import serial
import time
from collections import deque
import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as ds
import datetime as dt
#import sys
import matplotlib.dates as mdates

def get_data():
    # Request data
    arduino.write('r')
    time.sleep(0.1)
    result = arduino.readline()
    result = result.strip()
    temp,ir_temp = result.split(sep,2)
    return (float(temp),float(ir_temp))

def draw_graph(t,ir,Y,m,d,M,H,S,imgfile):

    t = np.array(t)
    ir = np.array(ir)

    times = np.zeros(len(Y))
    for i in range(0,len(Y)):
        times[i] = ds.date2num(dt.datetime(Y[i],m[i],d[i],H[i],M[i],S[i]))

    ok = np.where(ir > -50)
    t = t[ok]
    ir = ir[ok]
    times = times[ok]

    ok = np.where(t > -50)
    ok2 = np.where(t < 50)
    t = t[ok]
    t = t[ok2]
    ir = ir[ok]
#    ir = ir[ok2]
    times = times[ok]
    times2 = times[ok2]

    #xmin = times.min()
    xmax = times.max()+5/(24*60.0)
    xmin = xmax-1
    xFmt = mdates.DateFormatter('%H')
    hours = mdates.HourLocator()
    quarterHours = mdates.MinuteLocator(interval=15)

    width = 800/100.0
    height = 400/100.0
    
    fig = plt.figure(figsize=(width,height))
    ax = fig.add_subplot(111)

    ax.plot(times2,t,'r.',markersize=3.0)
    ax.plot(times,ir,'b.',markersize=3.0)
#    ax.plot(times,-1*(t-ir),'m')

    # format the ticks
    ax.xaxis.set_major_locator(hours)
    ax.xaxis.set_major_formatter(xFmt)
    ax.xaxis.set_minor_locator(quarterHours)
    ax.set_xlim(xmin,xmax)
    ax.grid(True)

    t_max = t.max()+1
    ir_max = ir.max()+1
#    plt.ylim(ymax=max(t_max,ir_max))
#    plt.ylim(ymin=max(t[-1],ir[-1])-40)
    #if t_min > -30 and ir_min > -30:
    #    plt.ylim(ymin=-30.0)
    
    leg = plt.legend(('Ambient','Sky'), loc=6)
    leg.get_frame().set_alpha(0.5)
    plt.ylabel('Temperature $^{\circ}$C')
    plt.xlabel('UTC Hour')
    ax.set_title('Sky and ambient temperature in Perkkaa')
    idxs = times >= (times[-1] - 5.0/(24*60.0))
    ir_grad5 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad5 = 5*ir_grad5[0]/(24*60) # Gradient deg C / 5 min
    idxs = times >= (times[-1] - 15.0/(24*60.0))
    ir_grad15 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad15 = 15*ir_grad15[0]/(24*60) # Gradient: deg C / 15 min
    idxs = times >= (times[-1] - 30.0/(24*60.0))
    ir_grad30 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad30 = 30*ir_grad30[0]/(24*60) # Gradient: deg C / 30 min
    idxs = times >= (times[-1] - 60.0/(24*60.0))
    ir_grad60 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad60 = 60*ir_grad60[0]/(24*60) # Gradient: deg C / 60 min
    idxs = times >= (times[-1] - 90.0/(24*60.0))
    ir_grad90 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad90 = 90*ir_grad90[0]/(24*60) # Gradient: deg C / 90 min
    idxs = times >= (times[-1] - 120.0/(24*60.0))
    ir_grad120 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad120 = 120*ir_grad120[0]/(24*60) # Gradient: deg C / 120 min
    idxs = times >= (times[-1] - 180.0/(24*60.0))
    ir_grad180 = np.polyfit(times[idxs], ir[idxs], 1)
    ir_grad180 = 180*ir_grad180[0]/(24*60) # Gradient: deg C / 180 min
    plt.figtext(0.91, 0.80, 'Latest:', color='k')
    plt.figtext(0.91, 0.75, str(round(t[-1],1))+' $^{\circ}$C',color='r')
    plt.figtext(0.91, 0.70, str(round(ir[-1],1))+' $^{\circ}$C',color='b')
    plt.figtext(0.91, 0.65, '$\Delta$: '+str(round(t[-1]-ir[-1],1))+' $^{\circ}$C',color='k', fontsize=9)
#    plt.figtext(0.91, 0.55, '$^{\circ}$C/(N min)', color='k',fontsize=9)
    plt.figtext(0.91, 0.55, 'Gradients:', color='k',fontsize=9)
    plt.figtext(0.91, 0.50, ' 5: '+str(round(ir_grad5,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.45, '15: '+str(round(ir_grad15,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.40, '30: '+str(round(ir_grad30,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.35, '60: '+str(round(ir_grad60,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.30, '90: '+str(round(ir_grad90,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.25, '2 h: '+str(round(ir_grad120,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.91, 0.20, '3 h: '+str(round(ir_grad180,1))+' $^{\circ}$C',color='b',fontsize=9)
    plt.figtext(0.65, 0.01, 'Updated: '+str(Y[-1])+'.'+str(m[-1]).zfill(2)+'.'+str(d[-1]).zfill(2)+' '+str(H[-1]).zfill(2)+'.'+str(M[-1]).zfill(2)+'.'+str(S[-1]).zfill(2), color='k')

#    ax.spines['top'].set_color('black')
#    ax.spines['bottom'].set_color('white')
#    ax.spines['left'].set_color('white')
    ax.spines['right'].set_color((1,1,1,0))
    for t in ax.xaxis.get_ticklines(): t.set_color('gray')
    for t in ax.yaxis.get_ticklines(): t.set_color('gray')

    plt.savefig(imgfile,format='png')


datadir = ''
imgfile = ''
dev = '/dev/ttyUSB0'
if len(sys.argv) > 1:
    dev = sys.argv[1]
speed = 38400
interval = 15
draw_every_Nth = 4
sep = ','

now = dt.datetime.utcnow()
one_day = dt.timedelta(days=1)
yesterday = now - one_day

datefile_yesterday = datadir+'/'+yesterday.strftime('%Y%m%d_cloud_sensor.txt')
datefile_today = datadir+'/'+now.strftime('%Y%m%d_cloud_sensor.txt')

n_vals = int(60*60*24/interval)

temps = deque(range(0), n_vals)
ir_temps = deque(range(0), n_vals)
Y = deque(range(0), n_vals)
m = deque(range(0), n_vals)
d = deque(range(0), n_vals)
H = deque(range(0), n_vals)
M = deque(range(0), n_vals)
S = deque(range(0), n_vals)

try:
    prev_data = np.loadtxt(datefile_yesterday, delimiter=',')
    if len(prev_data) > 10:
        for i in range(0,len(prev_data)):
            temps.append(prev_data[i,0].astype('float'))
            ir_temps.append(prev_data[i,1].astype('float'))
            Y.append(prev_data[i,2].astype('int'))
            m.append(prev_data[i,3].astype('int'))
            d.append(prev_data[i,4].astype('int'))
            H.append(prev_data[i,5].astype('int'))
            M.append(prev_data[i,6].astype('int'))
            S.append(prev_data[i,7].astype('int'))
except IOError:
    pass


try:
    prev_data = np.loadtxt(datefile_today, delimiter=',')
    if len(prev_data) > 10:
        for i in range(0,len(prev_data)):
            temps.append(prev_data[i,0].astype('float'))
            ir_temps.append(prev_data[i,1].astype('float'))
            Y.append(prev_data[i,2].astype('int'))
            m.append(prev_data[i,3].astype('int'))
            d.append(prev_data[i,4].astype('int'))
            H.append(prev_data[i,5].astype('int'))
            M.append(prev_data[i,6].astype('int'))
            S.append(prev_data[i,7].astype('int'))
except IOError:
    pass


# Open serial connection
arduino = serial.Serial(dev,speed)
time.sleep(2)

n = 0
while True:
    req_time = time.time()
    temp,ir_temp = get_data()
    n += 1

    temps.append(temp)
    ir_temps.append(ir_temp)
    stamp = dt.datetime.utcnow()
    Y.append(int(stamp.strftime("%Y")))
    m.append(int(stamp.strftime("%m")))
    d.append(int(stamp.strftime("%d")))
    H.append(int(stamp.strftime("%H")))
    M.append(int(stamp.strftime("%M")))
    S.append(int(stamp.strftime("%S")))

    datefile = datadir+'/'+stamp.strftime('%Y%m%d_cloud_sensor.txt')
    f = open(datefile,'a')

    data = str(temp)+','+str(ir_temp)+","+stamp.strftime("%Y,%m,%d,%H,%M,%S")
    #print data
    f.write(data+'\n')
    f.close()
    
    if n >= draw_every_Nth:
        draw_graph(temps,ir_temps,Y,m,d,M,H,S,imgfile)
        n = 0

    ready_time = time.time()

    time.sleep(interval+req_time-ready_time)
    

